/**
 * @file	VoteCode.cxx
 * @brief	vote code class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "VoteCode.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>
#include <Security/KeyManager.hxx>
#include <System/Encoding/Base64Encoder.h>
#include <System/Security/CRC32Calculator.hxx>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

static const size_t VOTE_CODE_SIZE = 16;

ISMTTLog* VoteCode::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Vote.VoteCode",
    GETTEXT_PACKAGE);

VoteCode::VoteCode()
{
}

VoteCode::~VoteCode()
{
}

std::string VoteCode::generateComposeCode(const Vote& vote)
{
    SafeBuffer code;
    SafeBuffer checksum;
    SafeBuffer composeCode;
    std::string composeCodeBase64;

    m_logger->Debug("Generate compose code");

    generateChecksum(vote, checksum);
    Base64Encoder::ConvertFromBase64(vote.getBallotID(), code);
    getComposeCode(code, checksum, composeCode);
    composeCodeBase64 = composeCode.GetBase64Data();

    m_logger->Debug("code = " + code.GetBase64Data());
    m_logger->Debug("checksum = " + checksum.GetBase64Data());
    m_logger->Debug("compose code = " + composeCodeBase64);

    return composeCodeBase64;
}

void VoteCode::generateChecksum(const Vote& vote, SafeBuffer& checksum)
{
    SafeBuffer buffer;
    std::string message;
    Vote::SSequence sSequence = vote.getSelections();
    std::vector<Selection> selection(sSequence.begin(), sSequence.end());
    std::vector<Selection>::iterator its;

    std::sort(selection.begin(), selection.end(), &VoteCode::sortSelectionVote);

    for (its = selection.begin(); its != selection.end(); ++its)
    {
        Selection::BSequence bSequence = its->getBallotOptions();
        std::vector<std::string> ballots(bSequence.begin(), bSequence.end());
        std::vector<std::string>::iterator itb;

        std::sort(ballots.begin(), ballots.end(), &VoteCode::sortBallotVote);

        for (itb = ballots.begin(); itb != ballots.end(); ++itb)
        {
            message = message.append(itb->c_str());
        }
    }

    buffer.AddData(message);
    Cryptography::MsgDigestUtil digest(buffer);

    digest.getDigest(checksum,
                     Cryptography::MsgDigestUtil::getDefaultVoteDigest());
}

void VoteCode::getComposeCode(const SafeBuffer& code, const SafeBuffer& checksum,
                              SafeBuffer& dataOut)
{
    SafeBuffer compose;

    compose.AddData(code);
    compose.AddData(checksum);
    cryptoCode(compose, dataOut);
}

void VoteCode::getVoteCode(const std::string& composeCode, SafeBuffer& dataOut)
{
    SafeBuffer buffer;

    decryptoCode(composeCode, buffer);
    dataOut.AddData(const_cast<unsigned char *>(buffer.GetData()), VOTE_CODE_SIZE);
}

void VoteCode::getChecksum(const std::string& composeCode, SafeBuffer& dataOut)
{
    SafeBuffer buffer;

    decryptoCode(composeCode, buffer);

    if (buffer.GetData() != NULL && buffer.GetSize() > VOTE_CODE_SIZE)
    {
        const unsigned char* data = buffer.GetData() + VOTE_CODE_SIZE;
        dataOut.AddData(const_cast<unsigned char *>(data),
                        (buffer.GetSize() - VOTE_CODE_SIZE));
    }
}

void VoteCode::getVoteCode(const Vote& vote, SafeBuffer& dataOut)
{
    getVoteCode(vote.getCode(), dataOut);
}

void VoteCode::getChecksum(const Vote& vote, SafeBuffer& dataOut)
{
    getChecksum(vote.getCode(), dataOut);
}

bool VoteCode::validateCode(const Vote& vote)
{
    SafeBuffer codeFromVote;
    SafeBuffer checksumFromVote;
    SafeBuffer generatedChecksum;
    bool isValid;

    m_logger->Debug("Validate code");
    getVoteCode(vote, codeFromVote);
    getChecksum(vote, checksumFromVote);
    generateChecksum(vote, generatedChecksum);

    isValid = (codeFromVote.GetBase64Data() == vote.getBallotID());
    isValid = isValid && (checksumFromVote == generatedChecksum);

    m_logger->Debug("session id    = " + vote.getSessionID());
    m_logger->Debug("ballot id     = " + vote.getBallotID());
    m_logger->Debug("vote code     = " + codeFromVote.GetBase64Data());
    m_logger->Debug("vote checksum = " + checksumFromVote.GetBase64Data());
    m_logger->Debug("checksum      = " + generatedChecksum.GetBase64Data());
    m_logger->Debug(std::string("Vote code is: ") +
                    (isValid ? "Valid" : "Invalid"));

    return isValid;
}

bool VoteCode::validateChecksum(const Vote& vote)
{
    SafeBuffer checksumFromVote;
    SafeBuffer generatedChecksum;
    bool isValid;

    getChecksum(vote, checksumFromVote);
    generateChecksum(vote, generatedChecksum);
    isValid = (checksumFromVote == generatedChecksum);
    m_logger->Debug(std::string("Vote Checksum is: ") +
                    (isValid ? "Valid" : "Invalid"));

    return isValid;
}

bool VoteCode::sortSelectionVote(const Selection& first, const Selection& second)
{
    std::string contestFirst(first.getContest());
    std::string contestSecond(second.getContest());

    return contestFirst < contestSecond;
}

bool VoteCode::sortBallotVote(const std::string& first, const std::string& second)
{
    return first < second;
}

void VoteCode::cryptoCode(const SafeBuffer& buffer, SafeBuffer& dataOut)
{
    try
    {
        Crypto obj;
        Smartmatic::System::SafeBuffer key;

        obj.init(Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, false);
        Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKey (
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, key );

        obj.encryptBufferToBufferAndCreateIV(buffer, dataOut, key.GetData());
    }
    catch (const Smartmatic::System::Exception::CryptoException& ex)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        m_logger->Debug(ss.str());
    }
    catch (...)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__;
        m_logger->Debug(ss.str());
    }
}

void VoteCode::decryptoCode(const std::string& composeCode, SafeBuffer& dataOut)
{
    try
    {
        SafeBuffer key;
        SafeBuffer data;
        Crypto obj;

        Base64Encoder::ConvertFromBase64(composeCode, data);
        obj.init(Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, false);

        Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKey(
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, key);

        obj.decryptBufferToBufferAndGetIV(data, dataOut, key.GetData());
    }
    catch (const Smartmatic::System::Exception::CryptoException& ex)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        m_logger->Debug(ss.str());
    }
    catch (...)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__;
        m_logger->Debug(ss.str());
    }
}


void VoteCode::encode(std::string& composeCode)
{
    unsigned int group = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCountGroupVoteCode();

    if (composeCode.length() > group && group > 0)
    {
        std::string character = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCharSeparatorVoteCode();
        std::string code;
        size_t parts = composeCode.length() / group - 1;

        for (size_t i = 0; i <= parts; ++i)
        {
            if (i != 0)
            {
                code.append(character);
            }

            code.append(composeCode.substr(0, group));
            composeCode.erase(0, group);
        }

        if (composeCode.length() > 0 && code.length() > 0)
        {
            code.append(character);
        }

        code.append(composeCode);
        composeCode = code;
    }
}

void VoteCode::decode(std::string& composeCode)
{
    unsigned int group = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCountGroupVoteCode();

    if (group > 0)
    {
        std::string find = SAESConfigurationManager::getInstance()->getVoteCodeConfig().getCharSeparatorVoteCode();
        composeCode = Smartmatic::System::Utils::StringUtils::replaceAll(composeCode, find, "");
    }
}
