/**
 * @file	SecretKey.h
 * @brief	Declaration of reader the secret key which contains the methods required for parse the encrypted file.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef SECRETKEY_H_
#define SECRETKEY_H_

#include <string>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <System/Exception/SecretKeyException.h>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Security
        {
            class Encryption;
        }

        namespace Common
        {
            namespace Security
            {
                class KeyManager;

                /**
                 * @class	SecretKey
                 * @brief	Reader which contains the functionalities for obtain and expose the secret key.
                 * @author	Cesar.bonilla
                 * @date	4/28/2011
                 */

                class SecretKey
                {

                    public:

                        /**
                         * @enum	KeyDerivAlgType
                         * @brief	Type of the key derivation algorithm
                         */
                        typedef enum
                        {
                            KEYDERIV_HMAC,	///< Enumerator that represent the selection of Hashed Message Authentication Code for the key derivation function.
                            KEYDERIV_DES, ///< Enumerator that represent the selection Data Encryption Standard for key derivation function.
                            KEYDERIV_3KEY3DES,	///< Enumerator that represent the selection of triple DES for the triple key derivation function.
                            KEYDERIV_2KEY3DES,	///< Enumerator that represent the selection of triple DES for the double key derivation function.
                            KEYDERIV_RC4	///< Enumerator that represent the selection of Rivest Cipher 4 for the key derivation function.

                        } KeyDerivAlgType;


                        // Lifecycle

                        /**
                         * @fn	SecretKey::SecretKey();
                         * @brief	Default constructor initializes an instance of the SecretKey class.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         */

                        SecretKey();

                        /**
                         * @fn	SecretKey::SecretKey(const std::string & filePath,
                         * 			const std::string & passPhrase) throw (System::Exception::SecretKeyException);
                         * @brief	  Constructor initializes an instance of the SecretKey class.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @exception	System::Exception::SecretKeyException	Thrown when a secret key error condition occurs.
                         * @param	filePath		Full pathname of the file.
                         * @param	passPhrase	The pass phrase.
                         * @param	votingCenterCode	The voting center code.
                         * @param	rnd     	The random string used to generate the salt.
                         */

                        SecretKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd )
                        throw ( System::Exception::SecretKeyException );

                        /**
                         * @fn	SecretKey::SecretKey (const SecretKey& other);
                         * @brief	Copy Constructor initializes an instance of the SecretKey class.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	other	The other.
                         */

                        SecretKey ( const SecretKey & other );

                        /**
                         * @fn	virtual SecretKey::~SecretKey();
                         * @brief	Finalizes the instance of the SecretKey class.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         */

                        virtual ~SecretKey();


                        // Operator overloading

                        /**
                         * @fn	SecretKey& SecretKey::operator=(const SecretKey& sk);
                         * @brief	Assignment operator.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	sk	The sk.
                         * @return	A shallow copy of this object.
                         */

                        SecretKey & operator= ( const SecretKey & sk );


                        // Public interface

                        /**
                         * @fn	std::string SecretKey::getKeyBase64 () const;
                         * @brief	Accessor to the file stored secret key.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	the key contained in the file in a base64 encoding format.
                         */

                        std::string getKeyBase64 () const;

                        /**
                         * @fn	void SecretKey::getKey (unsigned char*& key, int& keyLen);
                         * @brief	Accessor to the file stored secret key.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @param	key			out parameter. Is allocated with a new buffer containing the key bytes.
                         * @param	keyLen	out parameter. Will contain the key buffer length.
                         * ### return	keylen out parameter. Will contain the key buffer length.
                         */
                        void getKey ( unsigned char *& key, int & keyLen );

                        /**
                         * @fn	const unsigned char* SecretKey::getKey (int& keyLen) const;
                         * @brief	Accessor to the file stored secret key.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @param [in,out]	keyLen	out parameter. Is allocated with a new buffer containing the key bytes.
                         * @return	pointer to the key content.
                         */
                        const unsigned char * getKey ( int & keyLen ) const;

                        /**
                         * @fn	void clearPlainPassphrase ()
                         * @brief	Clear passphrase.
                         * @author	Yadickson Soto
                         * @date	08/05/2013
                         * @return	The plain passphrase.
                         */
                        void clearPlainPassphrase ()
                        {
                            m_Passphrase.clear();
                        }

                        /**
                         * @fn	const unsigned char* SecretKey::getPassphrase (size_t& bufferLen) const;
                         * @brief	accesor to the encrypted passphrase.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @param [in,out]	bufferLen	Mutable integer wich will contain the size of the buffer.
                         * @return	a buffer with the encrypted passphrase.
                         */
                        const unsigned char * getPassphrase ( size_t & bufferLen ) const;

                        /**
                         * @fn	void getSafeBuffer ( Smartmatic::System::SafeBuffer & buffer );
                         * @brief	Getter method: secret key binary buffer
                         * @author	Yadickson Soto
                         * @date	07/05/2013
                         * @param 	buffer result buffer
                         */
                        void getSafeBuffer ( Smartmatic::System::SafeBuffer & buffer );

                    protected:

                        /**
                         * @fn	const std::string SecretKey::getPlainPassphrase () const
                         * @brief	Accesor to the user provided passphrase.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The plain passphrase.
                         */
                        const std::string getPlainPassphrase () const
                        {
                            return m_Passphrase;
                        }

                    private:

                        friend class Smartmatic::SAES::Common::Security::KeyManager;
                        friend class Smartmatic::SAES::Security::Encryption;

                        // properties

                        std::string m_FilePath;   ///< Full pathname of the file
                        std::string m_Passphrase; ///< The passphrase
                        std::string m_votingCenterCode; ///< The voting center code
                        std::string m_rnd; ///< The random number used to generate the salt

                        bool m_IsFileEncrypted; ///< true if is file encrypted

                        Smartmatic::System::SafeBuffer mp_DecryptionKey;	///< The decryption key
                        Smartmatic::System::SafeBuffer mp_SecretKey;	///< The secret key
                        Smartmatic::System::SafeBuffer mp_PassphraseEncrypted; ///< The passphrase encrypted


                        // member methods

                        /**
                         * @fn	void SecretKey::mUnwrap() throw (Smartmatic::System::Exception::SecretKeyException);
                         * @brief	Extract the secret key contained in the file decrypting it if necessary.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @exception	Smartmatic::System::Exception::SecretKeyException	Thrown when a secret key error
                         * 																															condition occurs.
                         */

                        void mUnwrap() throw ( Smartmatic::System::Exception::SecretKeyException );

                        /**
                         * @fn	void SecretKey::mProcessInputPassphrase( std::string& passphrase , std::string& salt ,
                         * 			std::string sep_pattern ,
                         * 			int &iterations) throw (Smartmatic::System::Exception::SecretKeyException);
                         * @brief	Parse the string provided by the client to obtain the parameters needed by the
                         * 				cryptography algorithms involved in the extraction of the secret key.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @exception	Smartmatic::System::Exception::SecretKeyException	Thrown when a secret key error
                         * 																															condition occurs.
                         * @param	passphrase 	The passphrase.
                         * @param	salt			 	The salt.
                         * @param	sep_pattern	The separator pattern.
                         * @param	iterations 	The iterations.
                         */

                        void mProcessInputPassphrase (
                            std::string & passphrase
                            , std::string & salt
                            , std::string sep_pattern
                            , int & iterations )
                        throw ( Smartmatic::System::Exception::SecretKeyException );

                        /**
                         * @fn	void SecretKey::mObtainSymmetricKey( const std::string & passPhrase ,
                         * 			const Smartmatic::System::SafeBuffer& salt ,
                         * 			const Smartmatic::SAES::Common::Configuration::KeyDerivAlgType drv_alg ,
                         * 			const Smartmatic::SAES::Common::Configuration::DigestAlgType dgst_alg ,
                         * 			const int iterations) throw (Smartmatic::System::Exception::SecretKeyException);
                         * @brief	Derives PBE key from the passphrase provided by the user.
                         * This method only support to Hashed Message Authentication Code for key derivation function.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @exception	Smartmatic::System::Exception::SecretKeyException	Thrown when a secret key error
                         * 																															condition occurs.
                         * @param	passPhrase	The pass phrase.
                         * @param	salt				The salt.
                         * @param	drv_alg			The driver algorithm.
                         * @param	dgst_alg		The dgst algorithm.
                         * @param	iterations	The iterations.
                         */

                        void mObtainSymmetricKey (
                            const std::string & passPhrase
                            , const Smartmatic::System::SafeBuffer & salt
                            , const Smartmatic::SAES::Common::Configuration::KeyDerivAlgType drv_alg
                            , const Smartmatic::SAES::Common::Configuration::DigestAlgType dgst_alg
                            , const int iterations )
                        throw ( Smartmatic::System::Exception::SecretKeyException );

                        /**
                         * @fn	void SecretKey::mClearKey ( Smartmatic::System::SafeBuffer & key )
                         * @brief	securely free a key buffer resources.
                         * @author	Yadickson Soto
                         * @date	05/11/2012
                         * @param	key	 	pointer to the buffer to be erased.
                         */

                        void mClearKey ( Smartmatic::System::SafeBuffer & key );

                        /**
                         * @fn	void SecretKey::mAssignKey ( Smartmatic::System::SafeBuffer & keyIn,
                        		Smartmatic::System::SafeBuffer & keyOut );
                         * @brief	securely fills a key buffer with a new key value.
                         * @author	Yadickson Soto
                         * @date	05/11/2012
                         * @param	keyIn 	source buffer pointer to copy from.
                         * @param	keyOut	destination buffer.
                         */

                        void mAssignKey ( const Smartmatic::System::SafeBuffer & keyIn,
                                          Smartmatic::System::SafeBuffer & keyOut );

                        /**
                         * @fn	void SecretKey::mEncryptPassphrase (const std::string & passphrase) throw (Smartmatic::System::Exception::SecretKeyException);
                         * @brief	store provided plain phassphrase encrypted with the default encryption key.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @exception	Smartmatic::System::Exception::SecretKeyException	Thrown when a secret key error
                         * 																															condition occurs.
                         * @param	passphrase	passphrase.
                         */

                        void mEncryptPassphrase ( const std::string & passphrase )
                        throw ( Smartmatic::System::Exception::SecretKeyException );

                        /**
                         * @fn	void SecretKey::processKey(const std::string & filePath,
                         * 			const std::string & passPhrase) throw (Smartmatic::System::Exception::SecretKeyException);
                         * @brief	Method process file key.
                         * @author	Yadickson Soto
                         * @date	13/02/2012
                         * @exception	Smartmatic::System::Exception::SecretKeyException	Thrown when a secret key error
                         * 																															condition occurs.
                         * @param	filePath		file path.
                         * @param	passPhrase	The pass phrase.
                         * @paramvotingCenterCode	The voting center code.
                         * @param	rnd     	The random string used to generate the salt.
                         */
                        void processKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd )
                        throw ( Smartmatic::System::Exception::SecretKeyException );


                        static Smartmatic::Log::ISMTTLog * logger; ///< The reference pointer of a logger instance.
                };
            }
        }
    }
}

#endif /* SECRETKEY_H_ */
