/**
 * @file CheckDataManager.cxx
 * @brief Body class to check data and synchronize
 * @data 29/07/2015
 * @author Yadickson Soto
 */

#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/MemoryEventManager.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/LoginKeyManager.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <System/Runtime/ProcessLauncher.h>
#include <Log/SMTTLogManager.h>

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::Log;

CheckDataManager * CheckDataManager::instance = NULL;

CheckDataManager::CheckDataManager()
    : isCheckData(false), isSynchronize(false), isSafeEject(false), isWaitingMount(false)
{
}

CheckDataManager::~CheckDataManager()
{
    instance = NULL;
}

void CheckDataManager::startDetection()
{
    MemoryEventManager::getInstance()->startEmitSignal();
}

void CheckDataManager::stopDetection()
{
    MemoryEventManager::getInstance()->stopEmitSignal();
}

void CheckDataManager::checkInsertion( unsigned int numberOfDevices )
{
    if ( !MemoryEventManager::getInstance()->isCanEmitSignal() )
    {
        return;
    }

    BCRPolling::getInstance()->pause ();
    CardPolling::getInstance()->pause ();

    std::string domMedia = MemoryPathController::getInstance()->getDomMemoryBasePath();
    std::string flashMedia = MemoryPathController::getInstance()->getFlashMemoryBasePath();

    if ( !domMedia.empty() && !flashMedia.empty() && !isCheckData )
    {
        isCheckData = true;
        isWaitingMount = false;
        start ( 0 ); // Starting check data
    }
    else if ( numberOfDevices == 2 && !isWaitingMount )
    {
        isWaitingMount = true;
        MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::TWO_MEMORY);
    }
    else
    {
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::CHECK_MEMORY_ERROR );
        stopDetection();
    }
}

void CheckDataManager::checkExtraction ( unsigned int numberOfDevices )
{
    if ( !MemoryEventManager::getInstance()->isCanEmitSignal() )
    {
        return;
    }

    if ( numberOfDevices == 0 || isCheckData || isSynchronize || isWaitingMount )
    {
        BCRPolling::getInstance()->pause ();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::ERROR_MEMORY );
        stopDetection();
        BCRPolling::getInstance()->stop ();
    }
    else if ( isSafeEject && numberOfDevices == 1 )
    {
        isSafeEject = false;
        BCRPolling::getInstance()->resume();
        CardPolling::getInstance()->resume();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::SAFE_EJECT );
    }
    else
    {
        BCRPolling::getInstance()->pause ();
        MemoryEventManager::getInstance()->addMemoryEvent ( numberOfDevices == 1 ? MemoryEventManager::ONE_MEMORY : MemoryEventManager::TWO_MEMORY );
    }
}

void CheckDataManager::startSafeSynchronize()
{
    if ( !isSynchronize )
    {
        isSynchronize = true;

        BCRPolling::getInstance()->pause ();
        CardPolling::getInstance()->pause();
        start ( 0 ); // Starting synchronize
    }
}

void CheckDataManager::clearSynchronize()
{
    isSafeEject = false;
    isSynchronize = false;

    SMTTLogManager::getInstance()->shutdown();
    SMTTLogManager::getInstance()->Reconfigure();

    LoginKeyManager::getInstance()->clear();
    MachineOperationManager::getInstance()->restoreVoteInAllMemory();
    BCRPolling::getInstance()->resume();
    CardPolling::getInstance()->resume();
}

bool CheckDataManager::process ()
{
    if ( isCheckData )
    {
        checkDataDevice();
    }
    else if ( isSynchronize )
    {
        safeSynchronize();
    }

    return false;
}

void CheckDataManager::checkDataDevice()
{
    MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::TWO_MEMORY );
    DeviceTypeMonitor * device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_USBSTORAGEMOUNTED );

    if ( MachineOperationManager::getInstance()->checkData() )
    {
        clearSynchronize();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::CHECK_MEMORY_OK );
    }
    else if( device->getNumberOfDevices() == 2 )
    {
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::CHECK_MEMORY_ERROR );
        stopDetection();
    }
    else
    {
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::ERROR_MEMORY );
        stopDetection();
    }

    isCheckData = false;
}

void CheckDataManager::safeSynchronize ()
{
    DeviceTypeMonitor* device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(
        DeviceType::CONFIGURATION_USBSTORAGEMOUNTED);

    MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::START_SYNCHRONIZE);
    stopDetection();

    if (MachineOperationManager::getInstance()->synchronize())
    {
        startDetection();
        if (device->getNumberOfDevices() == 2)
        {
            isSafeEject = true;
            MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::END_SYNCHRONIZE);
        }
        else
        {
            MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::ERROR_MEMORY);
        }
    }
    else
    {
        isSafeEject = false;
        startDetection();
        MemoryEventManager::getInstance()->addMemoryEvent(
            device->getNumberOfDevices() == 2 ?
            MemoryEventManager::CHECK_MEMORY_ERROR :
            MemoryEventManager::ERROR_MEMORY);
        stopDetection();
    }

    isSynchronize = false;
}

CheckDataManager * CheckDataManager::getInstance()
{
    if ( instance == NULL )
    {
        instance = new CheckDataManager();
    }

    return instance;
}
