/**
 * @file MemoryMessageWindow.cxx
 * @brief Body class to show memory message on block window
 * @date 09/06/2015
 * @author Yadickson Soto
 */

#include "MemoryMessageWindow.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/DeviceTypeMonitorWindowConfigurationManager.hxx>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Configuration;

MemoryMessageWindow * MemoryMessageWindow::instance = NULL;
ISMTTLog * MemoryMessageWindow::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Windows.MemoryMessageWindow", GETTEXT_PACKAGE );

MemoryMessageWindow::MemoryMessageWindow()
    : PopUpMessageWindow()
{
    widget = AbstractFactoryWidget::createMemoryStateWidget();
    widget->getSignalEnd().connect ( sigc::mem_fun ( *this, &MemoryMessageWindow::QuitDialog ) );
    widget->getCancelSafeEject().connect ( sigc::mem_fun ( *this, &MemoryMessageWindow::onQuitDialog ) );

    Gdk::Color color ( "black" );
    setTitleColor ( color );
    setTitleLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->GetEmptyWindowTitleFormConfiguration() );

    pushStepWidget ( widget );
}

MemoryMessageWindow::~MemoryMessageWindow()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void MemoryMessageWindow::start()
{
    if ( DeviceTypeMonitorWindowConfigurationManager::getInstance()->isVisible() )
    {
        logger->Debug ( "start" );
        signalChange.disconnect();
        signalChange = MemoryEventManager::getInstance()->getMemorySignal().connect ( sigc::mem_fun ( *this, &MemoryMessageWindow::deviceChange ) );
    }
}

void MemoryMessageWindow::stop()
{
    logger->Debug ( "stop" );
    signalChange.disconnect();
}

void MemoryMessageWindow::deviceChange ( MemoryEventManager::MemoryEvent event )
{
    logger->Debug ( "device change" );

    switch ( event )
    {
        case MemoryEventManager::ONE_MEMORY:
            {
                Gdk::Color color ( "red" );
                setTitleColor ( color );
                getSignalBlock().emit();
                widget->showOneMemory();
            }
            break;

        case MemoryEventManager::TWO_MEMORY:
            {
                Gdk::Color color ( "black" );
                setTitleColor ( color );
                widget->showVerifyTwoMemory();
            }
            break;

        case MemoryEventManager::CHECK_MEMORY_ERROR:
            {
                Gdk::Color color ( "red" );
                setTitleColor ( color );
                getSignalBlock().emit();
                widget->showMemoryCheckError();
            }
            break;

        case MemoryEventManager::ERROR_MEMORY:
            {
                Gdk::Color color ( "red" );
                setTitleColor ( color );
                getSignalBlock().emit();
                widget->showAllMemoryRemoved();
            }
            break;

        case MemoryEventManager::START_SYNCHRONIZE:
            {
                Gdk::Color color ( "black" );
                setTitleColor ( color );
                widget->showSynchronize();
            }
            break;

        case MemoryEventManager::CHECK_MEMORY_OK:
            {
                Gdk::Color color ( "red" );
                setTitleColor ( color );
                getSignalResume().emit();
                QuitDialog();
            }

            return;

        case MemoryEventManager::END_SYNCHRONIZE:
            {
                Gdk::Color color ( "black" );
                setTitleColor ( color );
                widget->showSafeEject();
            }
            break;

        case MemoryEventManager::SAFE_EJECT:
            {
                Gdk::Color color ( "black" );
                setTitleColor ( color );
                getSignalAllow().emit();
                QuitDialog();
            }

            return;

        default:
            return;
    }

    refreshStepWidget ();
    ShowDialog();
}

void MemoryMessageWindow::onQuitDialog()
{
    getSignalResume().emit();
    QuitDialog();
}

sigc::signal<void> MemoryMessageWindow::getSignalBlock()
{
    return signalBlock;
}

sigc::signal<void> MemoryMessageWindow::getSignalAllow()
{
    return signalAllow;
}

sigc::signal<void> MemoryMessageWindow::getSignalResume()
{
    return signalResume;
}

MemoryMessageWindow * MemoryMessageWindow::getInstance()
{
    if ( !instance )
    {
        instance = new MemoryMessageWindow();
    }

    return instance;
}

void MemoryMessageWindow::showMemoryCheckError()
{
    Gdk::Color color ( "red" );
    setTitleColor ( color );
    getSignalBlock().emit();
    widget->showMemoryCheckError();
    refreshStepWidget ();
    ShowDialog();
    stop();
}

void MemoryMessageWindow::showNoMemoryError()
{
    Gdk::Color color ( "red" );
    setTitleColor ( color );
    getSignalBlock().emit();
    widget->showAllMemoryRemoved();
    refreshStepWidget ();
    ShowDialog();
    stop();
}
