/**
 * @file MsgDigestUtil.cpp
 * @brief Body class msg digest util
 * @date 01/04/2011
 * @author Marcel Cordovi
 */

#include "MsgDigestUtil.h"
#include <openssl/evp.h>
#include <Log/SMTTLogManager.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define MD5     "md5" ///< digest md5
#define SHA1    "sha1" ///< digest sha1
#define SHA224  "sha224" ///< digest sha224
#define SHA256  "sha256" ///< digest sha256
#define SHA384  "sha384" ///< digest sha384
#define SHA512  "sha512" ///< digest sha512

#define BUFFER_SIZE 256 ///< buffer size

#define _ERROR_SETTING_CHANNEL_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorSettingChannel") ///< error setting channel
#define _ERROR_READING_SECRET_KEY_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorReadingSecretKey") ///< erro reading secret key
#define _ERROR_READING_FILE_HASHING_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorReadingFileHashing") ///< error reading file hashing
#define _ERROR_UNKHOWN_MESSAGE_DIGEST_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorUnknownMessageDigest") ///< error unkhown digest

using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::Log;

ISMTTLog * MsgDigestUtil::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.MsgDigestUtil.", GETTEXT_PACKAGE );


MsgDigestUtil::MsgDigestUtil ( const Smartmatic::System::SafeBuffer & buffer )
    : buffer ( buffer )
{

}

MsgDigestUtil::~MsgDigestUtil()
{

}

void MsgDigestUtil::getDigest ( Smartmatic::System::SafeBuffer & result, DigestAlgType alg ) throw ( MessageDigestException )
{
    unsigned char * md_buffer = NULL;
    int mdbuf_len = 0;
    result.Clear();

    md_buffer = mPerformHashing ( mMapAlgName ( alg ), buffer.GetData(), buffer.GetSize(), mdbuf_len );
    result.AddData ( md_buffer, mdbuf_len );
    free ( md_buffer );
}

const char * MsgDigestUtil::mMapAlgName ( DigestAlgType alg )
{
    switch ( alg )
    {
        default:
        case DGSTUTIL_MD5:
            return MD5;

        case DGSTUTIL_SHA1:
            return SHA1;

        case DGSTUTIL_SHA224:
            return SHA224;

        case DGSTUTIL_SHA256:
            return SHA256;

        case DGSTUTIL_SHA384:
            return SHA384;

        case DGSTUTIL_SHA512:
            return SHA512;
    }
}

unsigned char * MsgDigestUtil::mPerformHashing ( const char * algName, const unsigned char * data, int dataLen, int & outBufSize ) throw ( MessageDigestException )
{
    EVP_MD_CTX mdctx;
    const EVP_MD * md;
    unsigned char md_value[EVP_MAX_MD_SIZE];
    unsigned int md_len;
    unsigned char * res_buf = NULL;

    md = EVP_get_digestbyname ( algName );

    if ( !md )
    {
        throw MessageDigestException ( _ ( _ERROR_UNKHOWN_MESSAGE_DIGEST_ ), N_ ( _ERROR_UNKHOWN_MESSAGE_DIGEST_ ), logger );
    }

    EVP_MD_CTX_init ( &mdctx );
    EVP_DigestInit_ex ( &mdctx, md, NULL );
    EVP_DigestUpdate ( &mdctx, data, dataLen );
    EVP_DigestFinal_ex ( &mdctx, md_value, &md_len );
    EVP_MD_CTX_cleanup ( &mdctx );
    outBufSize = md_len;
    res_buf = ( unsigned char * ) malloc ( md_len * sizeof ( char ) );

    memcpy ( res_buf, md_value, md_len );

    return res_buf;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultFileDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultFileNameDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultCertificateDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultOpeningCodeDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultSecretKeyDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultComposeKeyDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultGenerateKeyDigest()
{
    return DGSTUTIL_SHA1;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultVoteDigest()
{
    return DGSTUTIL_SHA256;
}
