package org.smartmatic.smartcard;

import javacard.framework.APDU;
import javacard.framework.Applet;
import javacard.framework.ISO7816;
import javacard.framework.ISOException;
import javacard.framework.OwnerPIN;
import javacard.framework.JCSystem;
import javacard.framework.Util;

import org.globalplatform.*;

public class Smartcard extends Applet {

    private OwnerPIN globalPin = null;

    private OwnerPIN passwordPin = null;
    private byte[] passPinBuffer = null;

    private byte[] password = null;

    /* secure channel */

    short responseLength = 0;
    private SecureChannel sc;

    /* data state fields */

    private byte[] data = null;
    private byte[] remaining_data = null;
    private short rem_data = 0;
    private short data_len = 0;
    private short data_offset = 0;
    private boolean write_opened = false;

    /* constraint constants */

    private final short MAX_DATA_LEN;
    private final byte MAX_GLOBAL_PIN_LEN;
    private final byte MAX_PASS_PIN_LEN;
    private final byte MAX_PASS_LEN;
    private final byte GLBL_PIN_RTYS;
    private final byte PASS_PIN_RTYS;

    /* APDU codes */

    final static byte VOTING_CLA = (byte) 0x80; // CLA for the voting card
						// applet
    final static byte SECURE_CLA = (byte) 0x84; // CLA for wrapped apdus

    final static byte VRFY_GLOBAL_PIN = (byte) 0xF1; // INS for VerifyGlobalPIN

    final static byte VRFY_PASS_PIN = (byte) 0xF2; // INS for verifyPaswordPIN
    final static byte SET_PASS_PIN = (byte) 0xF3; // INS for setPaswordPIN
    final static byte GET_PASS_PIN = (byte) 0xF4; // INS for getPaswordPIN

    final static byte SET_PASS = (byte) 0xF5; // INS for setPasword
    final static byte GET_PASS = (byte) 0xF6; // INS for getPasword

    final static byte SET_DATA_FILE = (byte) 0xF7; // INS for setDataFile
    final static byte OPEN_FILE = (byte) 0x01; // P1 for setDataFile
    final static byte WRITE_FILE = (byte) 0x02; // P1 for setDataFile
    final static byte ABORT_FILE = (byte) 0x03; // P1 for setDataFile
    final static byte COMMIT_FILE = (byte) 0x01; // P2 for setDataFile

    final static byte GET_DATA_FILE = (byte) 0xF8; // INS for getDataFile
    final static byte RESET_OFFSET = (byte) 0x01; // P1 for getDataFile

    final static byte GET_VERSION = (byte) 0x89; // INS for getVersion
    final static byte VER_MAJOR = (byte) 0x31;
    final static byte VER_MINOR = (byte) 0x32;
    final static byte VER_REVISION = (byte) 0x35;

    public Smartcard(byte[] bArray, short bOffset, byte bLength) {

	remaining_data = new byte[2];

	byte Li = bArray[bOffset];
	byte Lc = bArray[(byte) (bOffset+ Li + 1)]; 
	byte La = bArray[(byte) (bOffset+ Li + Lc + 2)]; 
	short app_data_offset = (short) (Li + Lc + 3); 

	MAX_GLOBAL_PIN_LEN = (byte) (La - 6);
	GLBL_PIN_RTYS = bArray[(short)(bOffset + app_data_offset)];
	MAX_PASS_PIN_LEN = bArray[(short) (bOffset + app_data_offset + 1)];
	PASS_PIN_RTYS = bArray[(short) (bOffset + app_data_offset + 2)];
	MAX_PASS_LEN = bArray[(short) (bOffset + app_data_offset + 3)];

	byte LadHigh = bArray[(short) (bOffset + app_data_offset + 4)];
	byte LadLow = bArray[(short) (bOffset + app_data_offset + 5)];

	MAX_DATA_LEN = Util.makeShort(LadHigh, LadLow);

	// PIN initialization
	
	globalPin = new OwnerPIN(GLBL_PIN_RTYS, MAX_GLOBAL_PIN_LEN);
	passwordPin = new OwnerPIN(PASS_PIN_RTYS, MAX_PASS_PIN_LEN);
	passPinBuffer = new byte[MAX_PASS_LEN];
	
	globalPin.update(bArray, (short) (bOffset + app_data_offset + 6),
		MAX_GLOBAL_PIN_LEN);

	// Data initialization
		
	password = new byte[MAX_PASS_LEN];
	data = new byte[MAX_DATA_LEN];

	register(bArray, (short) (bOffset + 1), bArray[bOffset]);
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {

	new Smartcard(bArray, bOffset, bLength);
    }

    public boolean select() {

	sc = GPSystem.getSecureChannel();
	globalPin.reset();
	passwordPin.reset();

	return true;
    }

    public void deselect() {

	sc.resetSecurity();
	globalPin.reset();
	passwordPin.reset();
    }

    public void process(APDU apdu) throws ISOException {

	byte[] buffer = apdu.getBuffer();

	if (buffer[ISO7816.OFFSET_CLA] == ISO7816.CLA_ISO7816
		&& buffer[ISO7816.OFFSET_INS] == ISO7816.INS_SELECT)
	    return;

	switch (buffer[ISO7816.OFFSET_INS]) {

	case VRFY_GLOBAL_PIN:

	    verifyGlobalPin(apdu);
	    break;

	case VRFY_PASS_PIN:

	    if (globalPin.isValidated())
		verifyPasswordPin(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case SET_PASS_PIN:

	    if (globalPin.isValidated())
		setPasswordPin(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case GET_PASS_PIN:

	    if (passwordPin.isValidated())
		getPasswordPin(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case SET_PASS:

	    if (passwordPin.isValidated() && globalPin.isValidated())
		setPassword(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case GET_PASS:

	    if (passwordPin.isValidated() && globalPin.isValidated())
		getPassword(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case SET_DATA_FILE:

	    if (passwordPin.isValidated() && globalPin.isValidated())
		setData(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;

	case GET_DATA_FILE:

	    if (passwordPin.isValidated() && globalPin.isValidated())
		getData(apdu);
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	    break;
	case GET_VERSION:
	    apdu.setOutgoing();
	    apdu.setOutgoingLength((short) 3);
	    buffer[0] = VER_MAJOR;
	    buffer[1] = VER_MINOR;
	    buffer[2] = VER_REVISION;
	    apdu.sendBytes((short) 0, (short) 3);

	    break;
	default:
	    processSCPcommands(apdu);
	}
    }

    void processSCPcommands(APDU apdu) {

	responseLength = sc.processSecurity(apdu);
	if (responseLength != 0)
	    apdu.setOutgoingAndSend((short) ISO7816.OFFSET_CDATA,
		    responseLength);
    }

    /* Command execution */

    private void verifyGlobalPin(APDU apdu) {

	byte[] buffer = apdu.getBuffer();
	byte bytesRead = (byte) apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = (byte) sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	if (bytesRead != MAX_GLOBAL_PIN_LEN)
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

	boolean verified = globalPin.check(buffer, ISO7816.OFFSET_CDATA,
		bytesRead);

	apdu.setOutgoing();

	if (verified)
	    buffer[ISO7816.OFFSET_CDATA] = 1;
	else
	    buffer[ISO7816.OFFSET_CDATA] = 0;

	apdu.setOutgoingLength((short) 1);
	apdu.sendBytes(ISO7816.OFFSET_CDATA, (short) 1);
    }

    private void verifyPasswordPin(APDU apdu) {

	byte[] buffer = apdu.getBuffer();
	byte bytesRead = (byte) apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = (byte) sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	if (bytesRead > MAX_PASS_PIN_LEN)
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

	boolean verified = passwordPin.check(buffer, ISO7816.OFFSET_CDATA,
		bytesRead);

	apdu.setOutgoing();

	if (verified)
	    buffer[ISO7816.OFFSET_CDATA] = 1;
	else
	    buffer[ISO7816.OFFSET_CDATA] = 0;

	apdu.setOutgoingLength((short) 1);
	apdu.sendBytes(ISO7816.OFFSET_CDATA, (short) 1);
    }

    private void setPasswordPin(APDU apdu) {

	byte[] buffer = apdu.getBuffer();

	short bytesRead = apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	if (bytesRead > MAX_PASS_PIN_LEN)
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

	JCSystem.beginTransaction();

	passwordPin.update(buffer, ISO7816.OFFSET_CDATA, (byte) bytesRead);
	Util.arrayFillNonAtomic(passPinBuffer, (short) 0, MAX_PASS_PIN_LEN,
		(byte) 0x00);
	Util.arrayCopy(buffer, ISO7816.OFFSET_CDATA, passPinBuffer, (byte) 0,
		bytesRead);
	resetAllData();

	JCSystem.commitTransaction();
    }

    private void getPasswordPin(APDU apdu) {

	byte[] buffer = apdu.getBuffer();
	short bytesRead = apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	apdu.setOutgoing();
	short wrapped_len = sc.wrap(passPinBuffer, (short) 0,
		(short) passPinBuffer.length);

	apdu.setOutgoingLength(wrapped_len);
	apdu.sendBytesLong(passPinBuffer, (short) 0, wrapped_len);
    }

    private void setPassword(APDU apdu) {

	byte[] buffer = apdu.getBuffer();

	short bytesRead = apdu.setIncomingAndReceive();
	short offset = 0;

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	if (bytesRead > MAX_PASS_LEN)
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

	JCSystem.beginTransaction();

	Util.arrayFillNonAtomic(data, (short) 0, MAX_DATA_LEN, (byte) 0x00);

	while (bytesRead != 0) {

	    Util.arrayCopy(buffer, ISO7816.OFFSET_CDATA, password,
		    (byte) offset, bytesRead);
	    offset += bytesRead;
	    bytesRead = apdu.receiveBytes(ISO7816.OFFSET_CDATA);

	    if ((short) (offset + bytesRead) > MAX_PASS_LEN)
		ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
	}

	JCSystem.commitTransaction();
    }

    private void getPassword(APDU apdu) {

	byte[] buffer = apdu.getBuffer();
	short bytesRead = apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA)
	    sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));

	apdu.setOutgoing();
	short wrapped_len = sc.wrap(password, (short) 0,
		(short) password.length);

	apdu.setOutgoingLength((short) wrapped_len);
	apdu.sendBytesLong(password, (short) 0, (short) wrapped_len);
    }

    private void setData(APDU apdu) {

	byte[] buffer = apdu.getBuffer();
	short bytesRead = apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	JCSystem.beginTransaction();

	if (buffer[ISO7816.OFFSET_P1] == OPEN_FILE) { // begin transaction
						      // operation

	    resetFileState();
	    write_opened = true;

	} else if (buffer[ISO7816.OFFSET_P1] == ABORT_FILE) { // abort
							      // transaction
							      // operation

	    if (write_opened) {

		resetFileState();
		write_opened = false;
		return;

	    } else {

		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED); // can't
									   // abort
									   // a
									   // not
									   // iniciated
									   // transaction
	    }

	} else if (buffer[ISO7816.OFFSET_P1] == WRITE_FILE) { // write operation

	    if (!write_opened)
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED); // deny
									   // writing
									   // without
									   // transaction
									   // opening
	} else {

	    ISOException.throwIt(ISO7816.SW_COMMAND_NOT_ALLOWED);
	}

	if ((short) (data_len + bytesRead) > MAX_DATA_LEN) {

	    resetFileState();
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
	}

	while (bytesRead != 0) {

	    Util.arrayCopy(buffer, ISO7816.OFFSET_CDATA, data, data_len,
		    bytesRead);
	    data_len += bytesRead;
	    bytesRead = apdu.receiveBytes(ISO7816.OFFSET_CDATA);

	    if ((short) (data_len + bytesRead) > MAX_DATA_LEN) {

		resetFileState();
		ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
	    }
	}

	if (buffer[ISO7816.OFFSET_P1] == WRITE_FILE
		&& buffer[ISO7816.OFFSET_P2] == COMMIT_FILE) {

	    if (write_opened)
		write_opened = false;
	    else
		ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);
	}

	JCSystem.commitTransaction();
    }

    private void getData(APDU apdu) {

	short remLen = 0;
	short datLen = 0;
	short outLen = 0;

	if (write_opened)
	    ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);

	byte[] buffer = apdu.getBuffer();
	short bytesRead = apdu.setIncomingAndReceive();

	if (buffer[ISO7816.OFFSET_CLA] == SECURE_CLA) {
	    bytesRead = sc.unwrap(buffer, (short) ISO7816.OFFSET_CLA,
		    (short) (bytesRead + 5));
	    bytesRead -= 5;
	}

	JCSystem.beginTransaction();

	if (buffer[ISO7816.OFFSET_P1] == RESET_OFFSET)
	    data_offset = 0;

	short resLen = apdu.setOutgoing();
	short data_req = (short)(resLen - 2); 
	
	if ( resLen > 2) {

		if ( data_req > (short)(data_len - data_offset) )
			ISOException.throwIt (ISO7816.SW_WRONG_LENGTH);

		rem_data = (short)(data_len - data_offset - data_req);
		Util.setShort(remaining_data, (short)0, rem_data);

		remLen = sc.wrap(remaining_data, (short)0, (short)2);
		datLen = sc.wrap(data, (short)data_offset, (short)data_req);
		outLen = (short)(remLen + datLen);
		
		apdu.setOutgoingLength( (short) (outLen));
		apdu.sendBytesLong(remaining_data, (short)0, (short)remLen);
		apdu.sendBytesLong(data, (short)data_offset, datLen);
		
		data_offset += data_req;
		
	} else if ( resLen == 2 ) {
		
		rem_data = (short)(data_len - data_offset);
		Util.setShort(remaining_data, (short)0, rem_data);
		
		remLen = sc.wrap(remaining_data, (short)0, (short)2);
		
		apdu.setOutgoingLength( (short) (2));
		apdu.sendBytesLong(remaining_data, (short)0, (short)remLen);
//			apdu.sendBytesLong(data, (short)data_offset, data_req);
	}
	else {
		
		ISOException.throwIt (ISO7816.SW_COMMAND_NOT_ALLOWED );
	}
	
	JCSystem.commitTransaction();
    }

    private void resetFileState() {

	data_len = 0;
	data_offset = 0;
	rem_data = 0;
	Util.arrayFillNonAtomic(data, (short) 0, MAX_DATA_LEN, (byte) 0x00);
    }

    private void resetAllData() {

	data_len = 0;
	data_offset = 0;
	rem_data = 0;
	Util.arrayFillNonAtomic(data, (short) 0, MAX_DATA_LEN, (byte) 0x00);
	Util.arrayFillNonAtomic(password, (short) 0, MAX_PASS_LEN, (byte) 0x00);
    }
}
