/**
 * 
 */
package com.smartmatic.security;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.CertStoreException;
import java.security.cert.CertificateException;
import java.util.Collection;
import java.util.Iterator;

import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;

import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSProcessable;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
import org.bouncycastle.cms.jcajce.JceKeyAgreeEnvelopedRecipient;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.mail.smime.SMIMEEnveloped;
import org.bouncycastle.mail.smime.SMIMEException;
import org.bouncycastle.mail.smime.SMIMESigned;
import org.bouncycastle.openssl.EncryptionException;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.util.Store;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * @author hector
 * 
 */
public class X7SEnvelopeHelper {


	/**
	 * Method that decrypts the information contained in a binary package data.
	 * 
	 * @param encryptedByteData
	 *            The message to be decrypted.
	 * @param privateKey
	 *            The private key used to decrypt the package.
	 * @return a binary representation of decrypted data.
	 * @throws EncryptionException
	 *             Thrown in case of any generic security error occurs.
	 */
	public static byte[] decrypt(byte[] encryptedByteData, PrivateKey privateKey)
			throws EncryptionException {
		byte[] decryptedByteData = new byte[] {};
		try {
			
			MimeBodyPart encryptedMimeBodyPart = new MimeBodyPart(
					new ByteArrayInputStream(encryptedByteData));
			SMIMEEnveloped enveloped = new SMIMEEnveloped(encryptedMimeBodyPart);

			RecipientInformationStore recipients = enveloped
					.getRecipientInfos();
			for (RecipientInformation recipientInfo : recipients.getRecipients()) {

				if (recipientInfo != null) {
					decryptedByteData = recipientInfo.getContent(new JceKeyAgreeEnvelopedRecipient(privateKey)
                            .setProvider(BouncyCastleProvider.PROVIDER_NAME));
				}
			}	
		} catch (Exception e) {
			throw new EncryptionException(
					"There was an IO problem during the en- or decryption process. See the stacktrace for details:" + e.getMessage(),
					e);
		}
		return decryptedByteData;
	}

	/**
	 * Check the package's validity
	 * 
	 * @param m
	 *            the binary signed data
	 * @return <value>true</value> if consistency of data is checked,
	 *         <value>false</value> otherwise.
	 * 
	 * @throws CMSException
	 *             Thrown when a generic error occurs during CMS sign process
	 * @throws NoSuchAlgorithmException
	 *             thrown when a particular cryptographic algorithm is requested
	 *             but is not available in the environment.
	 * @throws NoSuchProviderException
	 *             thrown when a particular security provider is requested but
	 *             is not available in the environment.
	 * @throws InvalidAlgorithmParameterException
	 *             exception for invalid or inappropriate algorithm parameters.
	 * @throws CertStoreException
	 *             Thrown when a generic error occurs during CMS sign process
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred
	 * @throws CertificateException 
	 * @throws OperatorCreationException 
	 * @throws SMIMEException 
	 * @throws MessagingException 
	 */
	@SuppressWarnings({ "rawtypes", "unused" })
	public static boolean verify(byte[] m) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException,
			CertStoreException, IOException, OperatorCreationException, CertificateException, MessagingException, SMIMEException {
		Security.addProvider(new BouncyCastleProvider());
		SMIMESigned s = new SMIMESigned(new MimeBodyPart(
				new ByteArrayInputStream(m)));
		boolean verified = false;

		Store                   certStore = s.getCertificates();
	    SignerInformationStore  signers = s.getSignerInfos();
		Collection              c = signers.getSigners();
		Iterator                it = c.iterator();
		  
		while (it.hasNext())
		{
	      SignerInformation   signer = (SignerInformation)it.next();
	      Collection          certCollection = certStore.getMatches(signer.getSID());

	      Iterator              certIt = certCollection.iterator();
	      X509CertificateHolder cert = (X509CertificateHolder)certIt.next();
	      PublicKey publicKey = new JcaX509CertificateConverter().getCertificate(cert).getPublicKey();
	  
	      if (signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(publicKey)))
	      {
	          verified=true;
	          break;
	      }   
		 }
		return verified;
	}

	/**
	 * 
	 * @param m
	 * @return
	 * @throws CMSException
	 * @throws NoSuchAlgorithmException
	 * @throws NoSuchProviderException
	 * @throws CertStoreException
	 * @throws IOException
	 * @throws CertificateException 
	 * @throws OperatorCreationException 
	 * @throws SMIMEException 
	 * @throws MessagingException 
	 */
	@SuppressWarnings("rawtypes")
	public static byte[] unsealedEnvelope(byte[] m) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException,
			CertStoreException, IOException, OperatorCreationException, CertificateException, MessagingException, SMIMEException {
		Security.addProvider(new BouncyCastleProvider());
		SMIMESigned s = new SMIMESigned(new MimeBodyPart(
				new ByteArrayInputStream(m)));
		
		Store                   certStore = s.getCertificates();
	    SignerInformationStore  signers = s.getSignerInfos();
		Collection              c = signers.getSigners();
		Iterator                it = c.iterator();
		  
		while (it.hasNext())
		{
	      SignerInformation   signer = (SignerInformation)it.next();
	      Collection          certCollection = certStore.getMatches(signer.getSID());

	      Iterator              certIt = certCollection.iterator();
	      X509CertificateHolder cert = (X509CertificateHolder)certIt.next();
	      PublicKey publicKey = new JcaX509CertificateConverter().getCertificate(cert).getPublicKey();
	      if (signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(publicKey)))
	      {
	    	  CMSProcessable signedContent = s.getSignedContent();
				byte[] originalContent = (byte[]) signedContent
						.getContent();
				return originalContent;
	      }   
		 }
		return null;
	}

	
	
	/**
	 * 
	 * @param key
	 * @param text
	 * @return
	 * @throws DocumentException
	 * @throws Base64DecodingException
	 */
	public static byte[] getRawData(String key, String text)
			throws DocumentException {
		Document doc = DocumentHelper.parseText(text);
		Element root = doc.getRootElement();
		Element pvElement = root.element(key);
		String rawData = pvElement.elementText("Data");

		byte[] decodedData = Base64.decodeBase64(rawData);
		return decodedData;
	}
}
