#!/bin/bash
#
# File: prepare-release.sh
#
# Date: Thu May 10, 13:30 EST 2018.
# Author: manuel.arguelles
#
# Revision: 3
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#

set -e

# Binaries
readonly gpg_bin=$(which gpg)
readonly find_bin=$(which find)
readonly tar_bin=$(which tar)
readonly mktemp_bin=$(which mktemp)
readonly readlink_bin=$(which readlink)
readonly unzip_bin=$(which unzip)
readonly xargs_bin=$(which xargs)
readonly sha256sum_bin=$(which sha256sum)
readonly sort_bin=$(which sort)
readonly rm_bin=$(which rm)
readonly test_bin=$(which test)

# Environment
readonly current_directory="$PWD"

function readProjectPath() {
  if [ -d "$1" ]; then
    project="$1"
  else
    read -p "Enter the path of the project: " project
  fi
}

function createPackageChecksum() {
  local -r package_name="$1"
  local -r tmp_dir="$(${mktemp_bin} -d)"
  local -r package_file="$(${readlink_bin} -e ${package_name})"

  cd "${tmp_dir}"

  if [[ "${package_name}" =~ \.tar\.gz$ ]]; then
    local -r checksum_file="${package_file%.tar.gz}-sha256sum.txt"
    local -r signature_file="${package_file%.tar.gz}-sha256sum.sig"
    $(${tar_bin} xzf "${package_file}")
  elif [[ "${package_name}" =~ \.tar$ ]]; then
    local -r checksum_file="${package_file%.tar}-sha256sum.txt"
    local -r signature_file="${package_file%.tar}-sha256sum.sig"
    $(${tar_bin} xf "${package_file}")
  elif [[ "${package_name}" =~ \.zip$ ]]; then
    local -r checksum_file="${package_file%.zip}-sha256sum.txt"
    local -r signature_file="${package_file%.zip}-sha256sum.sig"
    $(${unzip_bin} "${package_file}" >/dev/null)
  fi

  $(${find_bin} -type f -print0 | ${xargs_bin} -0 ${sha256sum_bin} | ${sort_bin} -k 2 > ${checksum_file})
  eval "${gpg_bin} -a -u \"${gpg_key_id}\" --output ${signature_file} --detach-sig ${checksum_file} >/dev/null"
  cd - >/dev/null
  $(${rm_bin} -rf ${tmp_dir})
}

function handlePackages() {
  local packages=()

  cd "${project}/Deliverables"

  while IFS=  read -r -d $'\0'; do
    packages+=("${REPLY}")
  done < <(${find_bin} \( \( -path ./Documentation -o -path ./ProofOfConcept \) -prune \) -o -name "*.tar.gz" -print0 -or -name "*.tar" -print0 -or -name "*.zip" -print0)

  for i in "${packages[@]}"; do
    createPackageChecksum "$i"
  done
}

function createFilesChecksum() {
  local -r directory="$1"
  local -r checksum_file="sha256sum.txt"
  local -r signature_file="sha256sum.sig"

  eval "${test_bin} -z \"$(${find_bin} ${directory} -maxdepth 1 -type f)\"" && return 0

  cd "${directory}"
  $(${find_bin} -path ./Documentation -prune -o -path ./ProofOfConcept -prune -o -type f -not -name ${checksum_file} -not -name "*.tar" -not -name "*.zip" -not -name "*.tar.gz" -print0 | ${xargs_bin} -0 ${sha256sum_bin} | ${sort_bin} -k 2 > ${checksum_file})
  eval "${gpg_bin} -a -u \"${gpg_key_id}\" --output ${signature_file} --detach-sig ${checksum_file} >/dev/null"
  cd - >/dev/null
}

function handleSecondLevelDirectories() {
  local directories=()

  cd "${project}/Deliverables"

  while IFS=  read -r -d $'\0'; do
    if [[ ! ( ${REPLY} =~ ^./Documentation || ${REPLY} =~ ^./ProofOfConcept ) ]]; then
      echo "Checking:  $REPLY"
      directories+=("${REPLY}")
    fi
  done < <(${find_bin} -mindepth 2 -maxdepth 2 -type d -print0)

  for i in "${directories[@]}"; do
    createFilesChecksum "$i"
  done
}

function readGpgKey() {
  echo "Available keys:"
  eval "${gpg_bin} --list-keys"
  read -p "Enter the key to use: " gpg_key_id
}

function importPrivateKeyOption() {
  printf "\nDo you want to import a private key (y/n)?"
  read -n 1 -p "Selection:" importInput
  if [ "$importInput" = "Y" ] || [ "$importInput" = "y" ]; then
    printf " Please input path of file with the key to import (full path to .asc or .gpg file)"
    read -p "Selection:" pgpFileInput
    set +e
    gpg --import $pgpFileInput
    set -e
    printf "These are the pgp keys you currently have in your default key ring"
    gpg --list-keys
  fi
}

function main() {
  # Main function
  readProjectPath "$@"
  importPrivateKeyOption
  readGpgKey
  handlePackages
  handleSecondLevelDirectories
  echo "All done!"
}

main $@
exit 0

