/**
 * AsymmetricCipherPoolFactory.java
 *
 * $Revision: 1.2 $ ($Date: 2009-11-05 23:37:50 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.asymmetric;

import java.security.PrivateKey;
import java.security.PublicKey;

import org.apache.commons.pool.BasePoolableObjectFactory;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * Class responsible to create Asymmetric ciphers. This class is used by the
 * AsymmetricCipher class to create a cipher object when its required. This is
 * te objects used in the pool by the class AsymmetricCipher
 * 
 * @version 1.0 (Feb 22, 2008)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class AsymmetricCipherPoolFactory extends BasePoolableObjectFactory {

	/**
	 * Enum that its used to show the posible key used by this asymmetric cipher
	 * pool
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum Mode {
		/**
		 * Private key type
		 */
		PRIVATE,
		/**
		 * Public key type
		 */
		PUBLIC
	}

	/**
	 * <p>
	 * Object that contains the private key used to encrypt/decrypt the messages
	 * that the server sends to another part of the system.
	 * </p>
	 */

	private PrivateKey privateKey;

	/**
	 * <p>
	 * Object that contains the public key used to encrypt/decrypt the messages
	 * that need to be sent do the server.
	 * </p>
	 */

	private PublicKey publicKey;

	/**
	 * Mode in which this asymmetric cipher will be used. The mode its used to
	 * know which key its used (either the public or the private)
	 */

	private Mode mode;

	/**
	 * Creates a AsymmetricCipherPoolFactory with a public key
	 * 
	 * @param publicKey
	 *            The key that will be used to create the ciphers
	 */

	public AsymmetricCipherPoolFactory(PublicKey publicKey) {
		this.publicKey = publicKey;
		this.mode = Mode.PUBLIC;
	}

	/**
	 * Creates a AsymmetricCipherPoolFactory with a private key
	 * 
	 * @param privateKey
	 *            The key that will be used to create the ciphers
	 */

	public AsymmetricCipherPoolFactory(PrivateKey privateKey) {
		this.privateKey = privateKey;
		this.mode = Mode.PRIVATE;
	}

	/**
	 * Method used to create a new object that will be used in the pool of
	 * ciphers
	 * 
	 * @return The Poolable Asymmetric Cipher using the key
	 * @throws CryptographicRuntimeException
	 *             if a problem in the encryption process occurs
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	@Override
	public PoolableAsymmetricCipher makeObject()
			throws CryptographicStaticException, CryptographicRuntimeException {
		if (this.mode == Mode.PUBLIC) {
			return new PoolableAsymmetricCipher(this.publicKey);
		} else {
			return new PoolableAsymmetricCipher(this.privateKey);
		}
	}

}
