/**
 * MessageDigestor.java
 *
 * $Revision: 1.10 $ ($Date: 2011-05-23 17:34:15 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */
package com.smartmatic.waffle.security.digest;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;

import com.smartmatic.waffle.lang.IExceptionInfo;
import com.smartmatic.waffle.security.lang.CryptographicProviderStaticException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * Class used to calculate the digest for messages.
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 5, 2009)
 */
public class MessageDigestor {

	/**
	 * <p>
	 * Field used when the digest algorithm that needs to be used is SHA-1. This
	 * is a private constant used to assemble this class with the Java Api for
	 * Digest Algorithm
	 * </p>
	 */

	private final static String DIGESTALGORITHMSHA = "SHA-1";

	/**
	 * <p>
	 * Field used when the digest algorithm that needs to be used is SHA-256.
	 * This is a private constant used to assemble this class with the Java Api
	 * for Digest Algorithm
	 * </p>
	 */

	private final static String DIGESTALGORITHMSHA256 = "SHA-256";

	/**
	 * <p>
	 * Field used when the digest algorithm that needs to be used is MD5. This
	 * is a private constant used to assemble this class with the Java Api for
	 * Digest Algorithm
	 * </p>
	 */

	private final static String DIGESTALGORITHMD5 = "MD5";

	/**
	 * Default mode that is used when the programmer does not specify any
	 * desired algorithm.
	 */

	private final static DigestModeEnum digestDefaultMode = DigestModeEnum.SHA_256;

	/**
	 * Field used to define that the digest algorithm that needs to be used is
	 * SHA-1. This is the field that outer classes have to use to comunicate to
	 * this class that the algorith desired is SHA-256
	 */

	public static final int MODE_SHA256 = 2;

	/**
	 * Field used to define that the digest algorithm that needs to be used is
	 * SHA-1. This is the field that outer classes have to use to comunicate to
	 * this class that the algorith desired is SHA-1
	 */

	public static final int MODE_SHA1 = 0;

	/**
	 * Field used to define that the digest algorithm that needs to be used is
	 * MD5. This is the field that outer classes have to use to comunicate to
	 * this class that the algorith desired is MD5
	 */

	public static final int MODE_MD5 = 1;

	/**
	 * Enum that contains the different modes supported as digest algorithms
	 * 
	 * @author itorres
	 */

	public enum DigestModeEnum {
		/**
		 * <p>
		 * Field used when the digest algorithm that needs to be used is SHA-1.
		 * This is a private constant used to assemble this class with the Java
		 * Api for Digest Algorithm
		 * </p>
		 */
		SHA_1(MessageDigestor.DIGESTALGORITHMSHA),
		/**
		 * <p>
		 * Field used when the digest algorithm that needs to be used is
		 * SHA-256. This is a private constant used to assemble this class with
		 * the Java Api for Digest Algorithm
		 * </p>
		 */
		SHA_256(MessageDigestor.DIGESTALGORITHMSHA256),
		/**
		 * <p>
		 * Field used when the digest algorithm that needs to be used is MD5.
		 * This is a private constant used to assemble this class with the Java
		 * Api for Digest Algorithm
		 * </p>
		 */
		MD_5(DIGESTALGORITHMD5);

		/**
		 * The algorithm that represents this mode
		 */

		private String algorithm;

		/**
		 * DigestModeEnum constructor
		 * 
		 * @param algorithm
		 *            algorithm to build the enum
		 */

		private DigestModeEnum(String algorithm) {
			this.algorithm = algorithm;
		}

		/**
		 * Getter of <code>algorithm</code> property.
		 * 
		 * @return <code>algorithm</code> property value.
		 * @see #algorithm
		 */
		public String getAlgorithm() {
			return this.algorithm;
		}

	}

	/**
	 * Method used to calculate the digest for a given message. The algorithm
	 * used for the digest operation is specified through the mode parameter.
	 * The values of the parameter mode are defined in the enum DigestModeEnum.
	 * The message digest is returned in a Base64 String. The Base64 conversion
	 * is done to keep all the application properties that could be seem by the
	 * user in a printable way.
	 * 
	 * @param message
	 *            String The message that will be digested
	 * @param mode
	 *            DigestModeEnum The mode (algorithm) that will be used to
	 *            digest the message
	 * @throws CryptographicProviderStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return String The Base64 digest of the message received as parameter
	 */

	public static String digest(String message, DigestModeEnum mode)
			throws CryptographicStaticException {
		try {
			MessageDigest messageDigest = MessageDigestor
					.getMessageDigest(mode);
			messageDigest.reset();
			messageDigest.update(message.getBytes());
			String result = new String(Base64.encodeBase64(
					messageDigest.digest(), false));
			return result;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM);
		}
	}

	/**
	 * Method used to calculate the digest for a given message. The algorithm
	 * used for the digest operation is the default mode specified through the
	 * constant MODE_DEFAULT defined in this class. The message digest is
	 * returned in a Base64 String. The Base64 conversion is done to keep all
	 * the application properties that could be seem by the user in a printable
	 * way.
	 * 
	 * @param message
	 *            String The message that will be digested
	 * @throws CryptographicProviderStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return String The Base64 digest of the message received as parameter
	 */

	public static String digest(String message)
			throws CryptographicStaticException {
		return digest(message, MessageDigestor.digestDefaultMode);
	}

	/**
	 * Auxiliar method that obtain the object in charge of calculate the digest
	 * of a given message.
	 * 
	 * @param mode
	 *            int The mode (algorithm) that will be used to digest the
	 *            message
	 * @throws NoSuchAlgorithmException
	 *             This exception is thrown when a particular cryptographic
	 *             algorithm is requested but is not available in the
	 *             environment
	 * @return MessageDigest The object that calculate the digest using the
	 *         algoritm specified
	 */

	private static MessageDigest getMessageDigest(DigestModeEnum mode)
			throws NoSuchAlgorithmException {
		MessageDigest messageDigest;
		messageDigest = MessageDigest.getInstance(mode.getAlgorithm());
		return messageDigest;
	}

	/**
	 * Method used to calculate the digest for a given message. The algorithm
	 * used for the digest operation is specified through the mode parameter.
	 * The values of the parameter mode are defined in the fields MODE_SHA1 and
	 * MODE_MD5. The message digest is returned in a byte array. This method
	 * does not calculte the Base64 of the digest.
	 * 
	 * @param message
	 *            String The message that will be digested
	 * @param mode
	 *            int The mode (algorithm) that will be used to digest the
	 *            message
	 * @throws CryptographicProviderStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return String The Base64 digest of the message received as parameter
	 */

	public static byte[] digest(byte[] message, int mode)
			throws CryptographicProviderStaticException {
		try {
			MessageDigest messageDigest = getMessageDigest(mode);
			messageDigest.reset();
			messageDigest.update(message);
			return messageDigest.digest();
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicProviderStaticException(
					new IExceptionInfo() {
						@Override
						public String getMessage() {
							return "saes.actlistener.security.MessageDigestor.WRONG_PROVIDER";
						}
					}, e);
		}
	}

	/**
	 * Method used to calculate the digest for a given message. The algorithm
	 * used for the digest operation is specified through the mode parameter.
	 * The values of the parameter mode are defined in the fields MODE_SHA1 and
	 * MODE_MD5. The message digest is returned in a Base64 String. The Base64
	 * conversion is done to keep all the application properties that could be
	 * seem by the user in a printable way.
	 * 
	 * @param message
	 *            String The message that will be digested
	 * @param mode
	 *            int The mode (algorithm) that will be used to digest the
	 *            message
	 * @throws CryptographicProviderStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return String The Base64 digest of the message received as parameter
	 */

	public static String digest(String message, int mode)
			throws CryptographicProviderStaticException {
		try {
			MessageDigest messageDigest = getMessageDigest(mode);
			messageDigest.reset();
			messageDigest.update(message.getBytes());
			String result = new String(Base64.encodeBase64(
					messageDigest.digest(), false));
			return result;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicProviderStaticException(
					new IExceptionInfo() {
						@Override
						public String getMessage() {
							return "saes.actlistener.security.MessageDigestor.WRONG_PROVIDER";
						}
					}, e);
		}
	}

	/**
	 * Method used to calculate the digest for a given message. The algorithm
	 * used for the digest operation is specified through the mode parameter.
	 * The values of the parameter mode are defined in the fields MODE_SHA1 and
	 * MODE_MD5. The message digest is returned in a Hex String. The Hex
	 * conversion is done to keep all the application properties that could be
	 * seem by the user in a printable way.
	 * 
	 * @param message
	 *            String The message that will be digested
	 * @param mode
	 *            int The mode (algorithm) that will be used to digest the
	 *            message
	 * @throws CryptographicProviderStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return String The Hex digest of the message received as parameter
	 */

	public static String digestHex(String message, int mode)
			throws CryptographicProviderStaticException {
		try {
			MessageDigest messageDigest = getMessageDigest(mode);
			messageDigest.reset();
			messageDigest.update(message.getBytes());
			String result = new String(Hex.encodeHex(messageDigest.digest(),
					false));
			return result.toLowerCase();
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicProviderStaticException(
					new IExceptionInfo() {
						@Override
						public String getMessage() {
							return "saes.actlistener.security.MessageDigestor.WRONG_PROVIDER";
						}
					}, e);
		}
	}

	/**
	 * Auxiliar method that obtain the object in charge of calculate the digest
	 * of a given message.
	 * 
	 * @param mode
	 *            int The mode (algorithm) that will be used to digest the
	 *            message
	 * @throws NoSuchAlgorithmException
	 *             This exception is thrown when a particular cryptographic
	 *             algorithm is requested but is not available in the
	 *             environment
	 * @return MessageDigest The object that calculate the digest using the
	 *         algoritm specified
	 */

	private static MessageDigest getMessageDigest(int mode)
			throws NoSuchAlgorithmException {
		MessageDigest messageDigest;
		if (mode == MessageDigestor.MODE_SHA1) {
			messageDigest = MessageDigest
					.getInstance(MessageDigestor.DIGESTALGORITHMSHA);
		} else if (mode == MessageDigestor.MODE_MD5) {
			messageDigest = MessageDigest
					.getInstance(MessageDigestor.DIGESTALGORITHMD5);
		} else {
			messageDigest = MessageDigest
					.getInstance(MessageDigestor.DIGESTALGORITHMSHA256);

		}
		return messageDigest;
	}
}