/**
 * SignXMLUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.security.sign;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Map;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.xml.sax.SAXException;

import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * The Class SignXMLUtil.
 */
public class SignXMLUtil {

	/**
	 * Sign xml.
	 * 
	 * @param PKCS12Bytes
	 *            the pKC s12 bytes
	 * @param caAlias
	 *            the ca alias
	 * @param caPassword
	 *            the ca password
	 * @param fileXml
	 *            the file xml
	 * @param signatureAlgorithm
	 *            the signature algorithm
	 * @return the string
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws InvalidAlgorithmParameterException
	 *             the invalid algorithm parameter exception
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws MarshalException
	 *             the marshal exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws TransformerException
	 *             the transformer exception
	 */
	public static void signXML(byte[] PKCS12Bytes, String caAlias,
			String caPassword, StringBuilder fileXml, String signatureAlgorithm)
			throws CryptographicStaticException, CryptographicRuntimeException,
			NoSuchAlgorithmException, InvalidAlgorithmParameterException,
			SAXException, IOException, ParserConfigurationException,
			MarshalException, XMLSignatureException, TransformerException {

		PrivateKey certificateCA = AsymmetricKeyManager.getPrivateKey(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias,
				caPassword);

		X509Certificate x509cert = AsymmetricKeyManager.getCertificate(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias);

		SignGenerator generator = new SignGenerator(x509cert, certificateCA,
				signatureAlgorithm);

		generator.signText(fileXml);
	}

	/**
	 * Sign detached xml.
	 * 
	 * @param PKCS12Bytes
	 *            the pKC s12 bytes
	 * @param caAlias
	 *            the ca alias
	 * @param caPassword
	 *            the ca password
	 * @param filesByUriBase
	 *            the files by uri base
	 * @param signatureAlgorithm
	 *            the signature algorithm
	 * @return the string
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws InvalidAlgorithmParameterException
	 *             the invalid algorithm parameter exception
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws MarshalException
	 *             the marshal exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws TransformerException
	 *             the transformer exception
	 */
	public static String signDetachedXMLFromCache(byte[] PKCS12Bytes,
			String caAlias, String caPassword,
			Map<String, List<Map.Entry<File, byte[]>>> filesByUriBase,
			String signatureAlgorithm) throws CryptographicStaticException,
			CryptographicRuntimeException, NoSuchAlgorithmException,
			InvalidAlgorithmParameterException, SAXException, IOException,
			ParserConfigurationException, MarshalException,
			XMLSignatureException, TransformerException {

		PrivateKey certificateCA = AsymmetricKeyManager.getPrivateKey(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias,
				caPassword);

		X509Certificate x509cert = AsymmetricKeyManager.getCertificate(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias);

		SignDetachedGenerator generator = new SignDetachedGenerator(x509cert,
				certificateCA, signatureAlgorithm);

		for (String baseUri : filesByUriBase.keySet()) {
			generator.addReferencesFromCache(baseUri,
					filesByUriBase.get(baseUri));
		}
		return generator.getSignedText();
	}

	/**
	 * Sign detached xml.
	 * 
	 * @param PKCS12Bytes
	 *            the pKC s12 bytes
	 * @param caAlias
	 *            the ca alias
	 * @param caPassword
	 *            the ca password
	 * @param filesByUriBase
	 *            the files by uri base
	 * @param signatureAlgorithm
	 *            the signature algorithm
	 * @return the string
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws InvalidAlgorithmParameterException
	 *             the invalid algorithm parameter exception
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws MarshalException
	 *             the marshal exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws TransformerException
	 *             the transformer exception
	 */
	public static String signDetachedXML(byte[] PKCS12Bytes, String caAlias,
			String caPassword, Map<String, List<File>> filesByUriBase,
			String signatureAlgorithm) throws CryptographicStaticException,
			CryptographicRuntimeException, NoSuchAlgorithmException,
			InvalidAlgorithmParameterException, SAXException, IOException,
			ParserConfigurationException, MarshalException,
			XMLSignatureException, TransformerException {

		PrivateKey certificateCA = AsymmetricKeyManager.getPrivateKey(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias,
				caPassword);

		X509Certificate x509cert = AsymmetricKeyManager.getCertificate(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias);

		SignDetachedGenerator generator = new SignDetachedGenerator(x509cert,
				certificateCA, signatureAlgorithm);

		for (String baseUri : filesByUriBase.keySet()) {
			generator.addReferences(baseUri, filesByUriBase.get(baseUri));
		}
		return generator.getSignedText();
	}
}
