/**
 * SymmetricKeyManager.java
 *
 * $Revision: 1.14 $ ($Date: 2011-04-06 23:22:02 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.symmetric;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.AlgorithmParameterGenerator;
import java.security.AlgorithmParameters;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidParameterSpecException;

import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;

import com.smartmatic.waffle.security.digest.MessageDigestor;
import com.smartmatic.waffle.security.lang.CryptographicProviderStaticException;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * <p>
 * Description: Class used to handle the symmetric keys that the application
 * needs to use. Through this class the storage of the keys is modular and
 * centralized.
 * </p>
 * <p>
 * This class allows the generation of keys and another specific parameters
 * needed to create a Symmetric Cipher. It is important to keep synchronized
 * this class with the asymmetric cipher. The synchronization is important
 * because this class generate keys that later are used by the symmetric cipher.
 * The keylengh of the keys generated in this class is configured through some
 * fields and must be a valid keylength supported by the algorithm defined in
 * the SymmetricCipher class
 * </p>
 * 
 * @version 1.0
 * @author Ithamar Torres
 */

public final class SymmetricKeyManager {

	/**
	 * Keystore type that will be used in the keystores where the keys are
	 * stored.
	 */

	public final static String KEYSTORE = "BKS";

	/**
	 * Algorithm used in the system. This is required by the key generation.
	 */

	private final static String KEYALG = "AES";

	/**
	 * Length in bits of the keys that will be generated using this class. This
	 * value has to be a valid key size of the algorithm specified in the field
	 * KEYALG.
	 */

	private final static int KEYLENGTH = 256;

	/**
	 * Method that load from a keystore the key that is stored with a given
	 * alias. Through this method this class interacts with the key repository
	 * and load the required key. This method use the default value for the
	 * keystore type KEYSTORE and null for the key password
	 * 
	 * @param inputStream
	 *            InputStream The input stream associated to the keystore
	 * @param keyStorePassword
	 *            String The password of the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 * @return SecretKey The secrete key loaded from the keystore
	 */

	public static SecretKey getSymmetricKey(InputStream inputStream,
			String keyStorePassword, String alias)
			throws CryptographicRuntimeException, CryptographicStaticException {
		return SymmetricKeyManager.getSymmetricKey(inputStream,
				keyStorePassword, alias, null, SymmetricKeyManager.KEYSTORE);
	}

	/**
	 * Method that load from a keystore the key that is stored with a given
	 * alias. Through this method this class interacts with the key repository
	 * and load the required key. This method use the default value for the
	 * keystore type KEYSTORE
	 * 
	 * @param inputStream
	 *            InputStream The input stream associated to the keystore
	 * @param keyStorePassword
	 *            String The password of the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyPassword
	 *            String used to set the key password.
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 * @return SecretKey The secrete key loaded from the keystore
	 */

	public static SecretKey getSymmetricKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPasword)
			throws CryptographicRuntimeException, CryptographicStaticException {
		return SymmetricKeyManager.getSymmetricKey(inputStream,
				keyStorePassword, alias, keyPasword,
				SymmetricKeyManager.KEYSTORE);
	}

	/**
	 * Method that load from a keystore the key that is stored with a given
	 * alias. Through this method this class interacts with the key repository
	 * and load the required key.
	 * 
	 * @param inputStream
	 *            InputStream The input stream associated to the keystore
	 * @param keyStorePassword
	 *            String The password of the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyStoreType
	 *            String used to set the keystore type.
	 * @param keyPassword
	 *            String used to set the key password.
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 * @return SecretKey The secrete key loaded from the keystore
	 */
	public static SecretKey getSymmetricKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPassword,
			String keyStoreType) throws CryptographicRuntimeException,
			CryptographicStaticException {
		try {
			KeyStore keyStore = KeyStore.getInstance(keyStoreType);
			keyStore.load(inputStream, keyStorePassword.toCharArray());
			SecretKey secretKey = null;
			if (keyPassword != null) {
				secretKey = (SecretKey) keyStore.getKey(alias,
						keyPassword.toCharArray());
			} else {
				secretKey = (SecretKey) keyStore.getKey(alias, null);
			}
			return secretKey;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM);
		} catch (UnrecoverableKeyException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.UNRECOVERABLE_ASYMMETRIC_KEY,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM);
		}
	}

	/**
	 * Method that load a SecretKey from his encoded form in a Base64 String.
	 * This method use the default Algorithm KEYALG
	 * 
	 * @param key
	 *            String the base 64 string representation of the key
	 * @return SecretKey The scret key used to encrypt / decrypt the information
	 */

	public static SecretKey getSymmetricKey(String key) {
		return SymmetricKeyManager.getSymmetricKey(key,
				SymmetricKeyManager.KEYALG);
	}

	/**
	 * Method that load a SecretKey from his encoded form in a Base64 String.
	 * 
	 * @param key
	 *            String the base 64 string representation of the key
	 * @param alg
	 *            String used to set the Algorithm
	 * @return SecretKey The scret key used to encrypt / decrypt the information
	 */

	public static SecretKey getSymmetricKey(String key, String alg) {
		Base64 codec = new Base64();
		byte[] decodedKey = codec.decode(key.getBytes());
		SecretKeySpec secretKey = new SecretKeySpec(decodedKey, alg);
		return secretKey;
	}

	/**
	 * Method that load a SecretKey from the raw byte array that is contained in
	 * the key. This method use the default Algorithm KEYALG
	 * 
	 * @param decodedKey
	 *            String the raw byte array from which is composed the key
	 * @return SecretKey The scret key used to encrypt / decrypt the information
	 */

	public static SecretKey getSymmetricKey(byte[] decodedKey) {
		return SymmetricKeyManager.getSymmetricKey(decodedKey,
				SymmetricKeyManager.KEYALG);
	}

	/**
	 * Method that load a SecretKey from the raw byte array that is contained in
	 * the key.
	 * 
	 * @param decodedKey
	 *            String the raw byte array from which is composed the key
	 * @param alg
	 *            String used to set the Algorithm
	 * @return SecretKey The scret key used to encrypt / decrypt the information
	 */

	public static SecretKey getSymmetricKey(byte[] decodedKey, String alg) {
		SecretKeySpec secretKey = new SecretKeySpec(decodedKey, alg);
		return secretKey;
	}

	/**
	 * Method that generate a new Secret Key of length KEYLENGTH. This method
	 * use default Algorithm KEYALG
	 * 
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return SecretKey The new key generated
	 */

	public static SecretKey generateSymmetricKey()
			throws CryptographicStaticException {
		return generateSymmetricKey(SymmetricKeyManager.KEYLENGTH);
	}

	/**
	 * Method that generate a new Secret Key of length defined by the caller.
	 * This methos use the default Algorithm KEYALG
	 * 
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return SecretKey The new key generated
	 */

	public static SecretKey generateSymmetricKey(int keyLength)
			throws CryptographicStaticException {
		return SymmetricKeyManager.generateSymmetricKey(keyLength,
				SymmetricKeyManager.KEYALG);

	}

	/**
	 * Method that generate a new Secret Key of length defined by the caller.
	 * 
	 * @param keyLength
	 *            Length used to generate the secret key.
	 * @param alg
	 *            Algorithm used to generate the secret key.
	 * 
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @return SecretKey The new key generated
	 */

	public static SecretKey generateSymmetricKey(int keyLength, String alg)
			throws CryptographicStaticException {
		try {
			KeyGenerator keyGenerator = KeyGenerator.getInstance(alg);
			keyGenerator.init(keyLength);

			SecretKey secretKey = keyGenerator.generateKey();

			return secretKey;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
	}

	/**
	 * Method that generates an aleatory inicialization vector that can be used
	 * in a Symmetric Cipher configured with a CBC ouput feedback mode. This
	 * method use the default Algorithm KEYALG
	 * 
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @return String The Inicialization Vector converted into a Base 64 String
	 */

	public static String generateIVector() throws CryptographicStaticException,
			CryptographicRuntimeException {
		return SymmetricKeyManager.generateIVector(SymmetricKeyManager.KEYALG);
	}

	/**
	 * Method that generates an aleatory inicialization vector that can be used
	 * in a Symmetric Cipher configured with a CBC ouput feedback mode
	 * 
	 * @param alg
	 *            The algorithm used to generate the IVector
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @return String The Inicialization Vector converted into a Base 64 String
	 */

	public static String generateIVector(String alg)
			throws CryptographicStaticException, CryptographicRuntimeException {
		String result = null;
		try {
			AlgorithmParameterGenerator parameterGenerator = AlgorithmParameterGenerator
					.getInstance(alg);
			AlgorithmParameters parameters = parameterGenerator
					.generateParameters();
			IvParameterSpec aESParameters = parameters
					.getParameterSpec(IvParameterSpec.class);
			byte[] iVector = aESParameters.getIV();
			result = new String(Base64.encodeBase64(iVector, false));
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidParameterSpecException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
		return result;
	}

	/**
	 * Method that generates an aleatory inicialization vector that can be used
	 * in a Symmetric Cipher configured with a CBC ouput feedback mode
	 * 
	 * @param alg
	 *            The algorithm used to generate the IVector
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @return String The Inicialization Vector converted into a Base 64 String
	 */

	public static byte[] generateIVectorByte()
			throws CryptographicStaticException, CryptographicRuntimeException {

		try {
			AlgorithmParameterGenerator parameterGenerator = AlgorithmParameterGenerator
					.getInstance(SymmetricKeyManager.KEYALG);
			AlgorithmParameters parameters = parameterGenerator
					.generateParameters();
			IvParameterSpec aESParameters = parameters
					.getParameterSpec(IvParameterSpec.class);

			return aESParameters.getIV();
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidParameterSpecException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
	}

	/***
	 * 
	 */
	public static byte[] generateIVector(byte[] array, int length)
			throws NoSuchAlgorithmException {
		byte[] info = new byte[array.length];
		for (int i = 0; i < array.length; i++) {
			boolean high = (array[i] % 2) == 0;
			info[i] = (byte) (high ? (array[i] & 0xF0) : (array[i] & 0x0F));
		}

		byte[] hash = MessageDigestor.digest(info, MessageDigestor.MODE_SHA1);
		byte[] genIV = new byte[length];
		if (hash.length < length) {
			int pasted = 0;
			while (pasted < length) {
				int lengthToCopy = Math.min(hash.length, length - pasted);
				System.arraycopy(hash, 0, genIV, pasted, lengthToCopy);
				pasted += lengthToCopy;
			}
		} else {
			System.arraycopy(hash, 0, genIV, 0, genIV.length);
		}
		return genIV;
	}

	/**
	 * Method that create a keystore given a secret key, the alias under the key
	 * is going to be stored and the passwoord of the keystore. This method use
	 * the default keystore type KEYSTORE. This method set the key password null
	 * 
	 * @param secretKey
	 *            The key that should be persisted in the keystore
	 * @param alias
	 *            Teh alias that will be used to store the key in the keystore
	 * @param keyStorePassword
	 *            The password of the keystore
	 * @return A output stream that contains the keystore
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws IOException
	 */

	public static KeyStore createKeystore(SecretKey secretKey, String alias,
			String keyStorePassword) throws KeyStoreException,
			NoSuchAlgorithmException, CertificateException, IOException {
		return SymmetricKeyManager.createKeystore(secretKey, alias,
				keyStorePassword, SymmetricKeyManager.KEYSTORE, null);
	}

	/**
	 * Method that create a keystore given a secret key, the alias under the key
	 * is going to be stored and the passwoord of the keystore. This method set
	 * the key password null
	 * 
	 * @param secretKey
	 *            The key that should be persisted in the keystore
	 * @param alias
	 *            Teh alias that will be used to store the key in the keystore
	 * @param keyStorePassword
	 *            The password of the keystore
	 * @param keyStoreType
	 *            String used to set the keystore type.
	 * @return A output stream that contains the keystore
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws IOException
	 */

	public static KeyStore createKeystore(SecretKey secretKey, String alias,
			String keyStorePassword, String keyStoreType)
			throws KeyStoreException, NoSuchAlgorithmException,
			CertificateException, IOException {
		return SymmetricKeyManager.createKeystore(secretKey, alias,
				keyStorePassword, keyStoreType, null);

	}

	/**
	 * Method that create a keystore given a secret key, the alias under the key
	 * is going to be stored and the passwoord of the keystore
	 * 
	 * @param secretKey
	 *            The key that should be persisted in the keystore
	 * @param alias
	 *            Teh alias that will be used to store the key in the keystore
	 * @param keyStorePassword
	 *            The password of the keystore
	 * @param keyStoreType
	 *            String used to set the keystore type.
	 * @param keyPassword
	 *            String used to set the password for the key
	 * @return A output stream that contains the keystore
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws IOException
	 */

	public static KeyStore createKeystore(SecretKey secretKey, String alias,
			String keyStorePassword, String keyStoreType, String keyPassword)
			throws KeyStoreException, NoSuchAlgorithmException,
			CertificateException, IOException {

		KeyStore keyStore = KeyStore.getInstance(keyStoreType);
		keyStore.load(null, keyStorePassword.toCharArray());
		if (keyPassword != null) {
			keyStore.setKeyEntry(alias, secretKey, keyPassword.toCharArray(),
					null);
		} else {
			keyStore.setKeyEntry(alias, secretKey, null, null);
		}
		return keyStore;
	}

	/**
	 * Method that allows to create a keystore and returns the input stream that
	 * contains the information inside the keystore
	 * 
	 * @param secretKey
	 * @return
	 */

	public static InputStream createKeystore(String alias,
			String keyStorePassword, SecretKey secretKey) {
		try {
			KeyStore keyStore = createKeystore(secretKey, alias,
					keyStorePassword);
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			keyStore.store(byteStream, keyStorePassword.toCharArray());
			ByteArrayInputStream inputStream = new ByteArrayInputStream(
					byteStream.toByteArray());
			return inputStream;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicProviderStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicProviderStaticException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM);
		}
	}

}